<?php
namespace AkTest\Wp\Plugin;

use Ak\Wp\ConfigProvider;
use Ak\Wp\Test\AbstractTestCase;
use Ak\Wp\Plugin\PluginLoader;

final class PluginLoaderTest extends AbstractTestCase
{
    public $configProviders = [
        ConfigProvider::class,
    ];

    public function testIsRegistered()
    {
        $container = $this->getContainer();

        $this->assertTrue($container->has('wp.plugin_loader'));
        $this->assertTrue($container->has(PluginLoader::class));

        $loader = $container->get('wp.plugin_loader');

        $this->assertInstanceOf(PluginLoader::class, $loader);
        $this->assertSame($loader, $container->get(PluginLoader::class));
    }

    public function testCanLoadPluginsFromConfig()
    {
        $container = $this->getContainer([
            'wp_plugins' => [
                'factories' => [
                    Mock\Plugin::class => function() {
                        $plugin = $this->createMock(Mock\Plugin::class);
                        $plugin->expects($this->once())
                            ->method('attach');
                        return $plugin;
                    },
                ],
            ],

            'wp' => [
                'plugins' => [
                    Mock\Plugin::class,
                    // Provjeravamo da se plugin učita samo jednom, uvijek
                    Mock\Plugin::class,
                ],
            ],
        ]);

        /** @var PluginLoader $loader */
        $loader = $container->get('wp.plugin_loader');
        $loader->loadPlugins();
    }

    public function testPluginCanBeDisabledFromConfig()
    {
        $factoryCalled = false;

        $container = $this->getContainer([
            'wp.plugins' => [
                'factories' => [
                    Mock\Plugin::class => function() use (&$factoryCalled) {
                        $factoryCalled = true;
                        $plugin = $this->createMock(Mock\Plugin::class);
                        $plugin->expects($this->never())
                            ->method('attach');
                        return $plugin;
                    },
                ],
            ],

            'wp' => [
                'plugins' => [
                    Mock\Plugin::class,
                    Mock\Plugin::class => false,
                ],
            ],
        ]);

        /** @var PluginLoader $loader */
        $loader = $container->get('wp.plugin_loader');
        $loader->loadPlugins();

        $this->assertFalse($factoryCalled);
    }

    public function testWillBeLoadedOnWordPressStart()
    {
        $loader = $this->createMock(PluginLoader::class);
        $loader->expects($this->once())
            ->method('register');

        $container = $this->getContainer([], [
            PluginLoader::class => $loader,
        ]);

        $this->runWordPress($container);
    }

    public function testLoaderWillAttachPlugins()
    {
        $plugin = $this->createMock(Mock\Plugin::class);
        $plugin->expects($this->once())
            ->method('attach');

        $container = $this->getContainer([
            'wp_plugins' => [
                'services' => [
                    'plugin' => $plugin,
                ],
            ],

            'wp' => [
                'plugins' => [
                    'plugin',
                ],
            ],
        ]);

        $this->runWordPress($container);
    }
}
